// Include required header files
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <stdio.h>
#include "main.h"
#include "protocol.h"

// Declare global variables
volatile unsigned char txBuffer[FRAME_SIZE] = {0, 0, 0, 0, 0};
volatile unsigned char txBufferIndex = 0;

void USART_init() {
  // Setup USART1

  // UBRR1 is the (16-bit) register for specifying the baud rate
  UBRR1 = 51; // 51 = 9600 baud, see datasheet
  
  // UCSR1B is a control register for the USART
  //   TXEN1 enables the TX circuitry
  //   TXCIE1 enables the TX-done interrupt
  //   UCSZ12 is the third bit that determines the serial frame length
  UCSR1B |= (1<<TXEN1)|(1<<TXCIE1)|(1<<UCSZ12);
  // UCSR1C is a control register for the USART
  //   UCSZ11 is the second bit that determines the serial frame length
  //   UCSZ10 is the first bit that determines the serial frame length
  UCSR1C |= (1<<UCSZ11)|(1<<UCSZ10);
  // UCSZ1n = 111 means a serial frame length of 9 bits
}

void LED_init() {
  // DDRE is the data direction register of port E
  //   A "1" means the IO pin is an output
  DDRE |= (1<<DDE6);
  // PORTE is the value register of port E
  //   A "1" means the IO pin is driven high
  PORTE &= ~(1<<PE6);
}

void txSendAddress(char Address) {
  // Send an address frame to the clients

  // Wait till transmit buffer is ready
  while ((UCSR1A & (1<<UDRE1)) == 0);

  // Signal Address frame
  // UCSR1B is a control register for the USART
  //   TXB81 is the 9-th bit of the frame (a "1" indicates an address frame)
  UCSR1B |= (1<<TXB81);
  // Writing a byte to UDR1 will start the communication
  UDR1 = Address;
}

void txSendData() {
  // Send a data frame to the clients

  // Wait till transmit buffer is ready
  while ((UCSR1A & (1<<UDRE1)) == 0);

  // Signal Data frame
  // UCSR1B is a control register for the USART
  //   TXB81 is the 9-th bit of the frame (a "0" indicates a data frame)
  UCSR1B &= ~(1<<TXB81);
  // Start from the beginning of the frame
  txBufferIndex = 0;
  // Writing a byte to UDR1 will start the communication
  UDR1 = txBuffer[txBufferIndex];
}

int main(void) {
  // Run the main program

  // Initialize the peripherals
  USART_init();
  LED_init();

  // Fill the frame with meaningful data
  txBuffer[FRAME_COMMAND] = COMMAND_LED;
  txBuffer[FRAME_DATA0] = 0x01;

  // Enable global interrupts
  sei();

  // Main loop
  while(1) {
    _delay_ms(1000);
    txSendAddress(0x00);
            _delay_ms(100);
    txBuffer[FRAME_COMMAND] = COMMAND_LED;
    txBuffer[FRAME_DATA0] = ON;
    txSendData();

    _delay_ms(1000);
    txSendAddress(0x01);
            _delay_ms(100);
    txBuffer[FRAME_COMMAND] = COMMAND_LED;
    txBuffer[FRAME_DATA0] = ON;
    txSendData();

    _delay_ms(1000);
    txSendAddress(0x02);
            _delay_ms(100);
    txBuffer[FRAME_COMMAND] = COMMAND_LED;
    txBuffer[FRAME_DATA0] = ON;
    txSendData();

    _delay_ms(1000);
    txSendAddress(0xFF);
            _delay_ms(100);
    txBuffer[FRAME_COMMAND] = COMMAND_LED;
    txBuffer[FRAME_DATA0] = OFF;
    txSendData();
  }
}

ISR(USART1_TX_vect) {
  // Interrupt vector for TX ready
  //   This interrupt fires when the last bit has been shifted out from the buffer
  //   and can be used to set the data direction in RS485 communication

  // Did we send an address?
  if ((UCSR1B & (1<<TXB81)) == 1) {
    // Yes, we sent an address, we are done
    PORTE |= (1<<PE6);  // this is the green led indicating we sent an address
  } else {
    // No, we sent data. Send the next byte if available
    PORTE &= ~(1<<PE6); // this is the green led indicating we send data
    if (txBufferIndex < (FRAME_SIZE - 1)) {
      //There are more bytes to send
      txBufferIndex++;
      UDR1 = txBuffer[txBufferIndex];
    } else {
      // DOne sending, leave a defined state
      txBufferIndex = FRAME_SIZE;
    }
  }
}